from r2.actions.action_dispatcher import ActionsDispatcher
from r2.helpers.constants import record_and_replay_ascii_logo


def main():
    print(record_and_replay_ascii_logo)
    actions_dispatcher = ActionsDispatcher()
    actions_dispatcher.process_application()


if __name__ == '__main__':
    main()



import json
import logging
from json.decoder import JSONDecodeError
from os import makedirs
from os.path import join, exists, abspath
from typing import Union, Any

import requests
import urllib3
from flask import Flask, request, abort, jsonify, Response
from requests.packages.urllib3.exceptions import InsecureRequestWarning

from r2.configuration import Configuration
from r2.core.package import Package
from r2.install import Installation

requests.packages.urllib3.disable_warnings(InsecureRequestWarning)
urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)


class FlaskServer:
    app = Flask(__name__)

    def __init__(self, target: str = None, package: str = 'default', save: bool = True, overwrite: bool = False,
                 mode: int = 1):
        if not target:
            logging.info("Working in REPLAY MODE")
        else:
            logging.info("Working in RECORD MODE")

        self.target = target
        self.package = package
        self.save = save
        self.overwrite = overwrite
        self.mode = mode

        self._download_dir = abspath(Installation.DOWNLOAD_DIR)
        self.abs_package_path = join(self._download_dir, package)
        if not exists(self.abs_package_path):
            makedirs(self.abs_package_path, exist_ok=True)

    def __call__(self):
        self.app.run()

    def serve(self):
        self.__call__()

    @staticmethod
    def dump_content(endpoint, response_body):
        package = Package(Configuration.read()["package_name"], True)
        package.save(endpoint, response_body)

    @staticmethod
    def load_content(endpoint):
        package = Package(Configuration.read()["package_name"])
        if not package.load(endpoint):
            return Response(status=404)
        else:
            return package

    @staticmethod
    @app.route('/favicon.ico')
    def favicon():
        return abort(404, description="Resource not found")

    @staticmethod
    @app.route('/', defaults={'path': ''}, methods=["GET", "POST", "PATCH", "DELETE"], strict_slashes=False)
    @app.route('/<path:path>', methods=["GET", "POST", "PATCH", "DELETE"], strict_slashes=False)
    def catch_traffic(path):
        headers = request.headers
        request_data = request.get_data().decode("utf-8")
        payload = None
        if request.content_type and request.content_type.lower() == 'application/json':
            try:
                payload = json.loads(request_data)
            except json.JSONDecodeError as err:
                print(err)
                payload = {}

        return FlaskServer.release_traffic(path, "GET", payload, headers)

    @staticmethod
    def release_traffic(path, method, payload, headers) -> Union[bytes, Any]:
        target = Configuration().read()["target"]
        mode = Configuration().read()["mode"]

        endpoint = f'{target}/{path}'
        if request.query_string:
            endpoint = f'{endpoint}?{request.query_string.decode("utf-8")}'

        # record mode == 0
        if mode == 0:
            return FlaskServer._record_mode(payload, method, endpoint, path, headers)
        elif mode == 1:
            return FlaskServer._replay_mode(path)
        else:
            raise NotImplementedError

    @staticmethod
    def _replay_mode(path):
        stored_target_response = FlaskServer.load_content(path)
        logging.info(f"Found stored response for path {path}")
        return stored_target_response

    @staticmethod
    def _record_mode(payload, method, endpoint, path, headers):
        # kwargs = {'verify': False,
        #           'headers': headers}
        kwargs = {'verify': False}
        if payload:
            if headers['Content-Type'] == "application/json":
                kwargs['data'] = json.dumps(payload)
            else:
                kwargs['data'] = payload

        request_method = {'GET': requests.get,
                          'POST': requests.post,
                          'PATCH': requests.patch,
                          'DELETE': requests.delete}[method.upper()]
        response = request_method(endpoint, **kwargs)

        status_code, response_body, headers = response.status_code, response.content.decode("utf-8"), response.headers
        logging.info(endpoint)
        try:
            target_response = json.loads(response_body)
            FlaskServer.dump_content(path, target_response)

            logging.info(f"Received a response from the target for the path {path}")
            return target_response
        except JSONDecodeError:
            logging.error("JSON response is corrupted")
            return jsonify({"r2error": "JSON response is corrupted"})


if __name__ == "__main__":
    f = FlaskServer(target="http://api.plos.org/", package='default')
    f()



from r2.actions.action import Action
from r2.configuration import Configuration
from r2.core.flask_server import FlaskServer


class ReplayAction(Action):
    ACTION = "replay"
    PARAM_NAME = "ACTION"

    def fill_parser_arguments(self):
        self.parser.add_argument("--package", help="Package name for the replay", type=str, nargs=1, default='default')

    def process_action(self, configuration):
        package = configuration.package

        Configuration(PACKAGE_NAME=package, MODE=1).save()
        FlaskServer(package=package, mode=1).serve()



import logging
from os import makedirs
from os.path import expanduser, join, exists


class Installation:
    HOME_DIR: str = join(expanduser('~'), '.r2')
    DOWNLOAD_DIR: str = join(HOME_DIR, "download")

    def __init__(self):
        self._create_default_directories()
        self._verifies_directories()

    def _create_default_directories(self):
        for d in (self.HOME_DIR, self.DOWNLOAD_DIR):
            makedirs(d, exist_ok=True)

    def _verifies_directories(self):
        for d in (self.HOME_DIR, self.DOWNLOAD_DIR):
            if not exists(d):
                logging.error(f"Can not create a {d} directory")
                raise

    @property
    def download_path(self):
        return self.DOWNLOAD_DIR



from argparse import ArgumentParser

from r2.actions.record.record import RecordAction
from r2.actions.replay.replay import ReplayAction


class ActionsDispatcher:
    ACTIONS_HANDLERS = [ReplayAction, RecordAction]

    def __init__(self):
        self.parser = ArgumentParser()
        subparsers = self.parser.add_subparsers()
        self.action_handlers = {action_handler.ACTION: action_handler(subparsers) for action_handler in
                                self.ACTIONS_HANDLERS}

    def process_application(self):
        configuration = self.parser.parse_args()
        if len(configuration.__dict__) == 0:
            self.parser.print_help()
            exit(0)
        self.action_handlers[configuration.ACTION].process_action(configuration)



import json
from dataclasses import dataclass
from os.path import join
from typing import Dict

from r2.install import Installation


@dataclass
class Configuration:
    TARGET: str = None
    PACKAGE_NAME: str = None
    MODE: int = None

    def _config_dict(self):
        return {
            "target": self.TARGET,
            "package_name": self.PACKAGE_NAME,
            "mode": self.MODE
        }

    @staticmethod
    def read():
        with open(Configuration.__config_path(), 'r') as config_file:
            return json.loads(config_file.read())

    @staticmethod
    def write(config_dict: Dict):
        with open(Configuration.__config_path(), 'w') as config_file:
            config_file.write(json.dumps(config_dict))

    def save(self):
        self.write(self._config_dict())

    @staticmethod
    def __config_path():
        return join(Installation.HOME_DIR, "config.r2")



import argparse
from abc import abstractmethod


class Action:
    ACTION = None
    PARAM_NAME = None

    def __init__(self, subparsers):
        self.parser = argparse.ArgumentParser(add_help=False)
        self.fill_parser_arguments()
        self.parser.set_defaults(**{self.PARAM_NAME: self.ACTION})
        subparsers.add_parser(self.ACTION, parents=[self.parser], formatter_class=argparse.RawTextHelpFormatter)

    def fill_parser_arguments(self):
        pass

    @abstractmethod
    def process_action(self, configuration):
        pass

    def get_config(self):
        self.parser.parse_args()


import setuptools

with open("../README.md", "r") as fh:
    long_description = fh.read()

with open("../VERSION", "r") as vr:
    version_number = vr.read()

with open("../requirements.txt", "r") as req:
    requirements = []
    for l in req.readlines():
        requirements.append(l.rstrip())

setuptools.setup(
    name="r2",
    version=version_number,
    author="Nex Sabre",
    author_email="nexsabre@protonmail.com",
    description="Software 'r2' is designed to record and play responses (to mock) from REST API services",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/NexSabre/r2",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    install_requires=requirements,
    entry_points={
        'console_scripts': [
            'r2 = r2.main:main'
        ],
    },
)



from r2.actions.action import Action
from r2.configuration import Configuration
from r2.core.flask_server import FlaskServer


class RecordAction(Action):
    ACTION = "record"
    PARAM_NAME = "ACTION"

    def fill_parser_arguments(self):
        self.parser.add_argument("target", help="Target service URI ", type=str, nargs=1)
        self.parser.add_argument("--package", help="Define a package name", type=str, nargs=1, default='default')
        self.parser.add_argument('-s', '--save', help="Save all incoming content the package folder", default=True)
        self.parser.add_argument('-o', '--overwrite', help="Overwrite all existing files in the package folder",
                                 default=False)

    def process_action(self, configuration):
        target = configuration.target[0]
        package = configuration.package

        Configuration(TARGET=target, PACKAGE_NAME=package, MODE=0).save()
        FlaskServer(target=target, package=package, save=configuration.save, overwrite=configuration.overwrite,
                    mode=0).serve()



import json
import logging
import os
from typing import AnyStr

from r2.install import Installation


class Package:
    def __init__(self, package_name: str = 'default', overwrite: bool = False):
        self._package_name = package_name
        self._overwrite = overwrite

        self._package_path = self._create_package_directory()[1]

    def _create_package_directory(self) -> (bool, AnyStr):
        abs_package_dir = os.path.join(Installation.DOWNLOAD_DIR, self._package_name)
        if os.path.exists(abs_package_dir):
            return True, abs_package_dir
        os.makedirs(abs_package_dir)
        return os.path.exists(abs_package_dir), abs_package_dir

    def save(self, endpoint, response_body):
        path, file = self._undress_endpoint(endpoint)
        self._ensure_path_exists(os.path.join(*(self._package_path, path)))

        with open(os.path.join(*(self._package_path, path, file)), 'w') as new_json:
            try:
                if isinstance(response_body, dict):
                    new_json.write(json.dumps(response_body))
                else:
                    new_json.write(response_body)
            except Exception as err:
                print(err)
                pass

    def load(self, endpoint):
        path, file = self._undress_endpoint(endpoint)
        path_exist, path_absolute = self._ensure_path_exists(os.path.join(*(self._package_path, path)))
        if not os.path.exists(path_absolute):
            return None
        if not os.path.isfile(os.path.join(path_absolute, file)):
            return None

        with open(os.path.join(path_absolute, file), 'r') as stored_response:
            json_response = None
            raw_response = stored_response.read()
            try:
                json_response = json.loads(raw_response)
            except Exception as err:
                logging.error(f"Can not convert raw_response into json format. Is file corrupted?\n{err}")
            finally:
                return json_response

    def _ensure_path_exists(self, path: str):
        if os.path.exists(path):
            return True, path
        os.makedirs(os.path.join(self._package_path, path), exist_ok=True)
        return os.path.exists(path), path

    @staticmethod
    def _undress_endpoint(endpoint: str) -> (str, str):
        l_endpoint = endpoint.split('/')
        if len(l_endpoint) > 1:
            path, filename = os.path.relpath(os.path.join(*l_endpoint[:-1])), l_endpoint[-1]
        else:
            path, filename = '', l_endpoint[-1]
        return path, filename
